"""Explore OAI-PMH user views
"""

from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.utils.rendering import render
from core_main_app.utils.view_builders import data as data_view_builder
from core_oaipmh_harvester_app.components.oai_record import (
    api as oai_record_api,
)


def data_detail(request):
    """Display data's detail for OAI-PMH

    Args:
        request:

    Returns:

    """
    try:
        record = oai_record_api.get_by_id(request.GET["id"], request.user)
        template = record.harvester_metadata_format.template

        data_object = {
            "record_id": record.id,
            "title": record.identifier,
            "content": record.content,
            "template": {
                "id": template.id if template is not None else "",
                "format": template.format if template is not None else "",
                "display_name": record.harvester_metadata_format.get_display_name(),
                "hash": record.harvester_metadata_format.hash,
            },
        }

        page_context = data_view_builder.build_page(data_object)

        page_context["assets"]["js"].extend(
            [
                {
                    "path": "core_explore_oaipmh_app/user/js/data/change_display.raw.js",
                    "is_raw": True,
                },
            ]
        )

        return data_view_builder.render_page(
            request,
            render,
            "core_main_app/user/data/detail.html",
            page_context,
        )
    except AccessControlError:
        error_message = "Access Forbidden"
        status_code = 403
    except Exception as exception:
        error_message = f"An error occurred: {str(exception)}"
        status_code = 400
    return render(
        request,
        "core_main_app/common/commons/error.html",
        context={"error": error_message, "status_code": status_code},
    )



""" Permission tests on ajax
"""

from django.test import RequestFactory

from core_explore_oaipmh_app.views.user.ajax import (
    get_data_source_list_oaipmh,
    update_data_source_list_oaipmh,
)
from core_main_app.utils.integration_tests.integration_base_test_case import (
    IntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from tests.fixtures.fixtures import AccessControlDataFixture


class TestGetDataSourceListOaipmh(IntegrationBaseTestCase):
    """Test Get Data Source List Oaipmh"""

    def setUp(self):
        """setUp"""

        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.anonymous = create_mock_user(user_id=None, is_anonymous=True)
        self.fixture = AccessControlDataFixture()
        self.fixture.insert_data()

    def test_an_anonymous_user_can_not_access_data_sources_of_a_user_query(
        self,
    ):
        """test_an_anonymous_user_can_not_access_data_sources_of_a_user_query"""

        request = self.factory.get("core_explore_oaipmh_app_get_data_sources")
        request.user = self.anonymous
        request.GET = {"id_query": str(self.fixture.query_user1.id)}
        response = get_data_source_list_oaipmh(request)
        self.assertEqual(response.status_code, 403)


class TestUpdateDataSourceListOaipmh(IntegrationBaseTestCase):
    """Test Update Data Source List Oaipmh"""

    def setUp(self):
        """setUp"""

        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1")
        self.anonymous = create_mock_user(user_id=None, is_anonymous=True)
        self.fixture = AccessControlDataFixture()
        self.fixture.insert_data()

    def test_an_anonymous_user_can_not_access_data_sources_of_a_user_query(
        self,
    ):
        """test_an_anonymous_user_can_not_access_data_sources_of_a_user_query"""

        request = self.factory.get(
            "core_explore_oaipmh_app_update_data_sources"
        )
        request.user = self.anonymous
        request.GET = {"id_query": str(self.fixture.query_user1.id)}
        response = update_data_source_list_oaipmh(request)
        self.assertEqual(response.status_code, 403)



""" Unit tests for Explore OAI-PMH REST API
"""

from unittest.mock import patch, MagicMock

from django.core import paginator as django_paginator
from django.test import SimpleTestCase, tag, override_settings

from core_explore_oaipmh_app.rest.query import views as query_views
from core_main_app.commons.exceptions import ApiError
from core_main_app.utils.pagination.mongoengine_paginator import (
    paginator as mongo_paginator,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import (
    create_mock_request,
)


class TestExecuteExecuteOaiPmhQuery(SimpleTestCase):
    """TestExecuteExecuteOaiPmhQuery"""

    @patch(
        "core_oaipmh_harvester_app.components.oai_registry.api.get_all_activated_registry"
    )
    @patch(
        "core_oaipmh_harvester_app.components.oai_record.api.execute_json_query"
    )
    def test_execute_local_query_returns_page_of_results(
        self, mock_execute_json_query, mock_get_all_activated_registry
    ):
        """test_execute_local_query_returns_page_of_results

        Returns:

        """
        # Arrange
        mock_user = create_mock_user(1)
        mock_query_data = {"query": {}}
        mock_request = create_mock_request(user=mock_user)
        mock_queryset = MagicMock()
        mock_execute_json_query.return_value = mock_queryset
        mock_activated_registries_response = MagicMock()
        mock_activated_registries_response.values_list.return_value = []
        mock_get_all_activated_registry.return_value = (
            mock_activated_registries_response
        )

        # Act
        page = query_views.execute_oaipmh_query(
            query_data=mock_query_data, page=1, request=mock_request
        )

        # Assert
        self.assertTrue(isinstance(page, django_paginator.Page))
        self.assertTrue(isinstance(page.paginator, django_paginator.Paginator))

    @patch(
        "core_oaipmh_harvester_app.components.oai_harvester_metadata_format.api.get_all_by_list_registry_ids"
    )
    @patch(
        "core_oaipmh_harvester_app.components.oai_registry.api.get_all_activated_registry"
    )
    @patch(
        "core_oaipmh_harvester_app.components.oai_record.api.execute_json_query"
    )
    def test_execute_oaipmh_query_with_params_returns_page_of_results(
        self,
        mock_execute_json_query,
        mock_get_all_activated_registry,
        mock_get_all_by_list_registry_ids,
    ):
        """test_execute_oaipmh_query_returns_page_of_results

        Returns:

        """
        # Arrange
        mock_user = create_mock_user(1)
        mock_query_data = {
            "query": {},
            "templates": '[{"id": 1}]',
            "options": '{"visibility": "public", "instance_id": "1"}',
            "order_by_field": "[]",
        }
        mock_request = create_mock_request(user=mock_user)
        mock_queryset = MagicMock()
        mock_execute_json_query.return_value = mock_queryset
        mock_activated_registries_response = MagicMock()
        mock_activated_registries_response.values_list.return_value = []
        mock_get_all_activated_registry.return_value = (
            mock_activated_registries_response
        )
        mock_all_by_list_ids_response = MagicMock()
        mock_all_by_list_ids_response.values_list.return_value = []
        mock_get_all_by_list_registry_ids.return_value = (
            mock_all_by_list_ids_response
        )
        # Act
        page = query_views.execute_oaipmh_query(
            query_data=mock_query_data, page=1, request=mock_request
        )

        # Assert
        self.assertTrue(isinstance(page, django_paginator.Page))
        self.assertTrue(mock_get_all_activated_registry.called)
        self.assertTrue(mock_get_all_by_list_registry_ids.called)

    @tag("mongodb")
    @override_settings(MONGODB_INDEXING=True)
    @patch(
        "core_oaipmh_harvester_app.components.oai_registry.api.get_all_activated_registry"
    )
    @patch(
        "core_oaipmh_harvester_app.components.oai_record.api.execute_json_query"
    )
    def test_execute_local_query_returns_page_of_mongo_results(
        self, mock_execute_json_query, mock_get_all_activated_registry
    ):
        """test_execute_local_query_returns_page_of_mongo_results

        Returns:

        """
        # Arrange
        mock_user = create_mock_user(1)
        mock_query_data = {"query": {}}
        mock_request = create_mock_request(user=mock_user)
        mock_queryset = MagicMock()
        mock_queryset.count.return_value = 0
        mock_execute_json_query.return_value = mock_queryset
        mock_activated_registries_response = MagicMock()
        mock_activated_registries_response.values_list.return_value = []
        mock_get_all_activated_registry.return_value = (
            mock_activated_registries_response
        )

        # Act
        page = query_views.execute_oaipmh_query(
            query_data=mock_query_data, page=1, request=mock_request
        )

        # Assert
        self.assertTrue(isinstance(page, django_paginator.Page))
        self.assertTrue(
            isinstance(page.paginator, mongo_paginator.MongoenginePaginator)
        )

    def test_execute_oaipmh_query_none_raises_api_error(self):
        """test_execute_oaipmh_query_none_raises_api_error

        Returns:

        """
        # Arrange
        mock_user = create_mock_user(1)
        mock_query_data = {}
        mock_request = create_mock_request(user=mock_user)

        # Act + Assert
        with self.assertRaises(ApiError):
            query_views.execute_oaipmh_query(
                query_data=mock_query_data, page=1, request=mock_request
            )


class TestFormatOaiPmhResults(SimpleTestCase):
    """TestFormatOaiPmhResults"""

    def test_format_oaipmh_empty_results_returns_empty_list(
        self,
    ):
        """test_format_oaipmh_empty_results_returns_empty_list

        Returns:

        """
        # Arrange
        mock_user = create_mock_user(1)
        mock_results = MagicMock()
        mock_results.object_list = []
        mock_request = create_mock_request(user=mock_user)

        # Act
        results = query_views.format_oaipmh_results(
            results=mock_results, request=mock_request
        )

        # Assert
        self.assertIsInstance(results, list)

    @override_settings(INSTALLED_APPS=[])
    def test_format_oaipmh_results_returns_list(
        self,
    ):
        """test_format_oaipmh_results_returns_list

        Returns:

        """
        # Arrange
        mock_user = create_mock_user(1)
        mock_data = MagicMock()
        mock_data.template_id = 1
        mock_harvester_metadata_format = MagicMock()
        mock_harvester_metadata_format.id = 1
        mock_harvester_metadata_format.name = "metadata_format_name"
        mock_data.harvester_metadata_format = mock_harvester_metadata_format
        mock_results = MagicMock()
        mock_results.object_list = [mock_data]
        mock_request = create_mock_request(user=mock_user)

        # Act
        results = query_views.format_oaipmh_results(
            results=mock_results, request=mock_request
        )

        # Assert
        self.assertIsInstance(results, list)
        self.assertTrue(len(results), 1)



""" OaiPmh Query api
"""

import core_explore_common_app.components.query.api as api_query
from core_explore_common_app.access_control.api import can_access
from core_explore_common_app.components.query.api import upsert
from core_main_app.access_control.decorators import access_control


@access_control(can_access)
def add_oaipmh_data_source(query, data_source, user):
    """Add an oaipmh data source to the query

    Args:
        query:
        data_source:
        user:

    Returns:

    """
    # the unique identifier for an oaipmh data source is its instance_id
    # because there is no constraint on the name
    data_source_found = False
    for data_source_item in query.data_sources:
        if (
            "instance_id" in data_source_item["query_options"]
            and data_source_item["query_options"]["instance_id"]
            == data_source["query_options"]["instance_id"]
        ):
            data_source_found = True

    if not data_source_found:
        # add data source to query if not present
        query.data_sources.append(data_source)
        # update query
        return upsert(query, user)


@access_control(can_access)
def remove_oaipmh_data_source(query, instance_id, user):
    """Remove an oaipmh data source to the query

    Args:
        query:
        instance_id:

    Returns:

    """
    data_source = None
    for data_source_item in query.data_sources:
        if (
            "instance_id" in data_source_item["query_options"]
            and data_source_item["query_options"]["instance_id"] == instance_id
        ):
            data_source = data_source_item

    if data_source:
        return api_query.remove_data_source(query, data_source, user)



""" Test user-side AJAX views
"""

from unittest import TestCase
from unittest.mock import patch, MagicMock
import json

from django.test import RequestFactory
from rest_framework import status

from core_explore_oaipmh_app.views.user.ajax import (
    update_data_source_list_oaipmh,
    change_data_display,
)
from core_main_app.utils.datetime import datetime_now
from core_oaipmh_harvester_app.components.oai_registry.models import (
    OaiRegistry,
)
from core_main_app.components.template.models import Template
from core_oaipmh_harvester_app.components.oai_record.models import OaiRecord
from core_oaipmh_harvester_app.components.oai_harvester_metadata_format.models import (
    OaiHarvesterMetadataFormat,
)
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.settings import DATA_SORTING_FIELDS, SERVER_URI
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.templatetags.xsl_transform_tag import (
    _render_xml_as_html,
)


class TestUpdateDataSourceListOaipmh(TestCase):
    """Test update_data_source_list_oaipmh function"""

    def setUp(self):
        """setUp"""
        self.factory = RequestFactory()
        self.view = "core_explore_oaipmh_app_update_data_sources"
        self.user = create_mock_user(user_id="1")

    def _send_request(self, data: dict = None):
        """_send_request"""
        request = self.factory.get(self.view)
        request.user = self.user
        request.GET = {} if not data else data

        return update_data_source_list_oaipmh(request)

    def test_id_query_none_returns_400(self):
        """test_id_query_none_returns_400"""
        self.assertEqual(
            self._send_request().status_code, status.HTTP_400_BAD_REQUEST
        )

    @patch("core_explore_oaipmh_app.views.user.ajax.api_query.get_by_id")
    def test_query_get_by_id_fails_returns_400(self, mock_query_get_by_id):
        """test_query_get_by_id_fails_returns_400"""
        mock_query_get_by_id.side_effect = Exception(
            "mock_query_get_by_id_exception"
        )
        self.assertEqual(
            self._send_request({"id_query": "mock_id_query"}).status_code,
            status.HTTP_400_BAD_REQUEST,
        )

    @patch("core_explore_oaipmh_app.views.user.ajax.api_query.get_by_id")
    def test_query_get_by_id_acl_error_returns_403(self, mock_query_get_by_id):
        """test_query_get_by_id_acl_error_returns_403"""
        mock_query_get_by_id.side_effect = AccessControlError(
            "mock_query_get_by_id_exception"
        )
        self.assertEqual(
            self._send_request({"id_query": "mock_id_query"}).status_code,
            status.HTTP_403_FORBIDDEN,
        )

    @patch("django.http.HttpRequest.build_absolute_uri")
    @patch("core_explore_oaipmh_app.views.user.ajax.api_query.get_by_id")
    def test_build_absolute_uri_fails_returns_400(
        self, mock_query_get_by_id, mock_build_absolute_uri
    ):
        """test_build_absolute_uri_fails_returns_400"""
        mock_query_get_by_id.return_value = "mock_query"
        mock_build_absolute_uri.side_effect = Exception(
            "mock_build_absolute_uri_exception"
        )
        self.assertEqual(
            self._send_request({"id_query": "mock_id_query"}).status_code,
            status.HTTP_400_BAD_REQUEST,
        )

    @patch(
        "core_explore_oaipmh_app.views.user.ajax.oai_registry_api.get_by_id"
    )
    @patch("django.http.HttpRequest.build_absolute_uri")
    @patch("core_explore_oaipmh_app.views.user.ajax.api_query.get_by_id")
    def test_registry_get_by_id_fails_returns_400(
        self,
        mock_query_get_by_id,
        mock_build_absolute_uri,
        mock_oai_registry_get_by_id,
    ):
        """test_registry_get_by_id_fails_returns_400"""
        mock_query_get_by_id.return_value = "mock_query"
        mock_build_absolute_uri.return_value = "mock_url_instance"
        mock_oai_registry_get_by_id.side_effect = Exception(
            "mock_oai_registry_get_by_id"
        )
        self.assertEqual(
            self._send_request({"id_query": "mock_id_query"}).status_code,
            status.HTTP_400_BAD_REQUEST,
        )

    @patch(
        "core_explore_oaipmh_app.views.user.ajax.oaipmh_query_api.add_oaipmh_data_source"
    )
    @patch(
        "core_explore_oaipmh_app.views.user.ajax.oai_registry_api.get_by_id"
    )
    @patch("django.http.HttpRequest.build_absolute_uri")
    @patch("core_explore_oaipmh_app.views.user.ajax.api_query.get_by_id")
    def test_data_source_capabilities_added_when_linked_records_installed(
        self,
        mock_query_get_by_id,
        mock_build_absolute_uri,
        mock_oai_registry_get_by_id,
        mock_add_oaipmh_data_source,
    ):
        """test_data_source_capabilities_added_when_linked_records_installed"""
        mock_oai_registry = MagicMock()
        mock_url_instance = "mock_url_instance"
        mock_query_get_by_id.return_value = "mock_query"
        mock_build_absolute_uri.return_value = mock_url_instance
        mock_oai_registry_get_by_id.return_value = mock_oai_registry

        self._send_request(
            {"id_query": "mock_id_query", "to_be_added": "true"}
        )
        mock_add_oaipmh_data_source.assert_called_with(
            "mock_query",
            {
                "name": mock_oai_registry.name,
                "url_query": SERVER_URI,
                "authentication": {"auth_type": "session", "params": {}},
                "order_by_field": ",".join(DATA_SORTING_FIELDS),
                "query_options": {"instance_id": str(mock_oai_registry.id)},
                "capabilities": {},
            },
            self.user,
        )

    @patch(
        "core_explore_oaipmh_app.views.user.ajax.oaipmh_query_api.add_oaipmh_data_source"
    )
    @patch(
        "core_explore_oaipmh_app.views.user.ajax.oai_registry_api.get_by_id"
    )
    @patch("core_explore_oaipmh_app.views.user.ajax.api_query.get_by_id")
    def test_data_source_capabilities_empty_when_linked_records_not_installed(
        self,
        mock_query_get_by_id,
        mock_oai_registry_get_by_id,
        mock_add_oaipmh_data_source,
    ):
        """test_data_source_capabilities_empty_when_linked_records_not_installed"""
        mock_oai_registry = MagicMock()
        mock_query_get_by_id.return_value = "mock_query"
        mock_oai_registry_get_by_id.return_value = mock_oai_registry

        self._send_request(
            {"id_query": "mock_id_query", "to_be_added": "true"}
        )
        mock_add_oaipmh_data_source.assert_called_with(
            "mock_query",
            {
                "name": mock_oai_registry.name,
                "url_query": SERVER_URI,
                "authentication": {"auth_type": "session", "params": {}},
                "order_by_field": ",".join(DATA_SORTING_FIELDS),
                "query_options": {"instance_id": str(mock_oai_registry.id)},
                "capabilities": {},
            },
            self.user,
        )

    @patch(
        "core_explore_oaipmh_app.views.user.ajax.oaipmh_query_api.add_oaipmh_data_source"
    )
    @patch(
        "core_explore_oaipmh_app.views.user.ajax.oai_registry_api.get_by_id"
    )
    @patch("core_explore_oaipmh_app.views.user.ajax.api_query.get_by_id")
    def test_add_oaipmh_data_source_fails_returns_400(
        self,
        mock_query_get_by_id,
        mock_oai_registry_get_by_id,
        mock_add_oaipmh_data_source,
    ):
        """test_add_oaipmh_data_source_fails_returns_400"""
        mock_oai_registry = MagicMock()
        mock_query_get_by_id.return_value = "mock_query"
        mock_oai_registry_get_by_id.return_value = mock_oai_registry
        mock_add_oaipmh_data_source.side_effect = Exception(
            "mock_add_oaipmh_data_source_exception"
        )

        self.assertEqual(
            self._send_request(
                {"id_query": "mock_id_query", "to_be_added": "true"}
            ).status_code,
            status.HTTP_400_BAD_REQUEST,
        )

    @patch(
        "core_explore_oaipmh_app.views.user.ajax.oaipmh_query_api.remove_oaipmh_data_source"
    )
    @patch(
        "core_explore_oaipmh_app.views.user.ajax.oai_registry_api.get_by_id"
    )
    @patch("core_explore_oaipmh_app.views.user.ajax.api_query.get_by_id")
    def test_remove_oaipmh_data_source_called_when_not_to_be_added(
        self,
        mock_query_get_by_id,
        mock_oai_registry_get_by_id,
        mock_remove_oaipmh_data_source,
    ):
        """test_remove_oaipmh_data_source_called_when_not_to_be_added"""
        mock_oai_registry = MagicMock()
        mock_query_get_by_id.return_value = "mock_query"
        mock_oai_registry_get_by_id.return_value = mock_oai_registry

        self._send_request(
            {"id_query": "mock_id_query", "to_be_added": "false"}
        )
        mock_remove_oaipmh_data_source.assert_called_with(
            "mock_query",
            None,
            self.user,
        )

    @patch(
        "core_explore_oaipmh_app.views.user.ajax.oaipmh_query_api.remove_oaipmh_data_source"
    )
    @patch(
        "core_explore_oaipmh_app.views.user.ajax.oai_registry_api.get_by_id"
    )
    @patch("core_explore_oaipmh_app.views.user.ajax.api_query.get_by_id")
    def test_remove_oaipmh_data_source_fails_returns_400(
        self,
        mock_query_get_by_id,
        mock_oai_registry_get_by_id,
        mock_remove_oaipmh_data_source,
    ):
        """test_remove_oaipmh_data_source_fails_returns_400"""

        mock_oai_registry = MagicMock()
        mock_query_get_by_id.return_value = "mock_query"
        mock_oai_registry_get_by_id.return_value = mock_oai_registry
        mock_remove_oaipmh_data_source.side_effect = Exception(
            "mock_remove_oaipmh_data_source_exception"
        )

        self.assertEqual(
            self._send_request(
                {"id_query": "mock_id_query", "to_be_added": "false"}
            ).status_code,
            status.HTTP_400_BAD_REQUEST,
        )

    @patch(
        "core_explore_oaipmh_app.views.user.ajax.oaipmh_query_api.remove_oaipmh_data_source"
    )
    @patch(
        "core_explore_oaipmh_app.views.user.ajax.oai_registry_api.get_by_id"
    )
    @patch("core_explore_oaipmh_app.views.user.ajax.api_query.get_by_id")
    def test_success_returns_200(
        self,
        mock_query_get_by_id,
        mock_oai_registry_get_by_id,
        mock_remove_oaipmh_data_source,
    ):
        """test_success_returns_200"""

        mock_oai_registry = MagicMock()
        mock_query_get_by_id.return_value = "mock_query"
        mock_oai_registry_get_by_id.return_value = mock_oai_registry
        mock_remove_oaipmh_data_source.return_value = None

        self.assertEqual(
            self._send_request(
                {"id_query": "mock_id_query", "to_be_added": "false"}
            ).status_code,
            status.HTTP_200_OK,
        )


class TestChangeDataDisplayOaipmh(TestCase):
    """Test change_data_display function"""

    def setUp(self):
        """setUp"""
        self.factory = RequestFactory()
        self.view = "core_explore_oaipmh_app_change_data_display"
        self.user = create_mock_user(user_id="1")

    def _send_request(self, data: dict = None):
        """_send_request"""
        request = self.factory.post(self.view)
        request.user = self.user
        request.POST = {} if not data else data

        return change_data_display(request)

    def test_id_record_none_returns_400(self):
        """test_id_record_none_returns_400"""
        self.assertEqual(
            self._send_request().status_code, status.HTTP_400_BAD_REQUEST
        )

    @patch("core_explore_oaipmh_app.views.user.ajax.oai_record_api.get_by_id")
    def test_change_data_display_acl_error_returns_403(
        self, mock_record_get_by_id
    ):
        """test_change_data_display_acl_error_returns_403"""
        mock_record_get_by_id.side_effect = AccessControlError(
            "mock_query_get_by_id_exception"
        )
        self.assertEqual(
            self._send_request({"record_id": "mock_id_query"}).status_code,
            status.HTTP_403_FORBIDDEN,
        )

    @patch("core_explore_oaipmh_app.views.user.ajax.oai_record_api.get_by_id")
    def test_change_data_display_returns_200(self, mock_record_get_by_id):
        """test_change_data_display_returns_200"""
        _oai_record = _create_oai_record()
        mock_record_get_by_id.return_value = _oai_record
        response = self._send_request({"record_id": "1"})
        response_html = json.loads(response.content.decode())
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response_html["template"],
            _get_change_display_data_result(_oai_record),
        )


def _create_oai_record():
    """Get an OaiRecord object.

    Returns:
        OaiRecord instance.

    """
    oai_record = OaiRecord()
    _set_oai_record_fields(oai_record)

    return oai_record


def _set_oai_record_fields(oai_record):
    """Set OaiRecord fields.

    Args:
        oai_record:

    Returns:
        OaiRecord with assigned fields.

    """
    oai_record.identifier = "oai:test/id.0006"
    oai_record.last_modification_date = datetime_now()
    oai_record.deleted = False
    oai_record.harvester_metadata_format = OaiHarvesterMetadataFormat()
    oai_record.registry = OaiRegistry()
    oai_record.harvester_metadata_format.template = Template(id=1)
    oai_record.xml_content = "<test><message>Hello</message></test>"

    return oai_record


def _get_change_display_data_result(oai_record):
    """_get_change_display_data_result

    Args:
        oai_record:
        request:

    Returns:
        transformed content.

    """

    return _render_xml_as_html(
        "test",
        xml_content=oai_record.xml_content,
        template_id=oai_record.harvester_metadata_format.template.id,
        template_hash=oai_record.harvester_metadata_format.template.hash,
    )



""" Ajax User core explore OAI-PMH
"""

import json

from django.http import HttpResponseForbidden
from django.http.response import HttpResponseBadRequest, HttpResponse
from django.shortcuts import render
from django.utils.html import escape

import core_explore_common_app.components.query.api as api_query
from core_explore_common_app.components.abstract_query.models import (
    Authentication,
    DataSource,
)
from core_explore_oaipmh_app.components.query import api as oaipmh_query_api
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.settings import DATA_SORTING_FIELDS, SERVER_URI
from core_main_app.templatetags.xsl_transform_tag import (
    render_xml_as_html_detail,
)
from core_oaipmh_harvester_app.components.oai_record import (
    api as oai_record_api,
)
from core_oaipmh_harvester_app.components.oai_registry import (
    api as oai_registry_api,
)


def get_data_source_list_oaipmh(request):
    """Ajax method to fill the list of data sources.

    Args:
        request:

    Returns:

    """
    try:
        id_query = request.GET.get("id_query", None)

        if id_query is None:
            return HttpResponseBadRequest(
                "Error during loading data sources from oaipmh search."
            )

        # Get query from id
        query = api_query.get_by_id(id_query, request.user)
        instance_list = oai_registry_api.get_all_activated_registry(
            order_by_field="name"
        )
        item_list = []
        for instance_item in instance_list:
            checked = False
            # compare instance with existing data source in query
            # in order to know if they have to be checked
            for data_source_item in query.data_sources:
                if "instance_id" in data_source_item[
                    "query_options"
                ] and data_source_item["query_options"]["instance_id"] == str(
                    instance_item.id
                ):
                    checked = True

            # update the result item list for the context
            item_list.extend(
                [
                    {
                        "instance_id": instance_item.id,
                        "instance_name": instance_item.name,
                        "is_checked": checked,
                    }
                ]
            )

        # Here, data sources are instances
        context_params = dict()
        context_params["instances"] = item_list

        # return context
        context = {}
        context.update(request)
        context.update(context_params)
        return render(
            request,
            "core_explore_oaipmh_app/user/data_sources/list-content.html",
            context,
        )

    except AccessControlError:
        return HttpResponseForbidden()
    except Exception as exception:
        return HttpResponseBadRequest(
            f"Error during loading data sources from oaipmh search: {escape(str(exception))}"
        )


def update_data_source_list_oaipmh(request):
    """Ajax method to update query data sources in data base.

    Args:
        request:

    Returns:

    """
    try:
        id_query = request.GET.get("id_query", None)
        id_instance = request.GET.get("id_instance", None)
        added = request.GET.get("to_be_added", None)
        to_be_added = json.loads(added) if added is not None else False

        # Get query from id
        if id_query is None:
            return HttpResponseBadRequest(
                "Error during data source selection."
            )

        query = api_query.get_by_id(id_query, request.user)
        instance = oai_registry_api.get_by_id(id_instance)
        if to_be_added:
            # Instance have to be added in the query as a data source
            authentication = Authentication(auth_type="session")
            data_source = DataSource(
                name=instance.name,
                url_query=SERVER_URI,
                authentication=authentication,
                order_by_field=",".join(DATA_SORTING_FIELDS),
                query_options={"instance_id": str(instance.id)},
            )

            oaipmh_query_api.add_oaipmh_data_source(
                query, data_source, request.user
            )
        else:
            oaipmh_query_api.remove_oaipmh_data_source(
                query, id_instance, request.user
            )

        return HttpResponse()

    except AccessControlError:
        return HttpResponseForbidden()
    except Exception as exception:
        return HttpResponseBadRequest(
            f"Error during data source selection: {escape(str(exception))}"
        )


def change_data_display(request):
    """Change data display

    Args:
        request:

    Returns:
    """
    try:
        # get xslt id
        xsl_transformation_id = request.POST.get("xslt_id", None)
        # get oai_record
        record = oai_record_api.get_by_id(
            request.POST.get("data_id"), request.user
        )
        # get template id
        template_id = record.harvester_metadata_format.template.id
        # get template hash
        template_hash = record.harvester_metadata_format.template.hash
        # get xml content
        xml_content = record.xml_content

        # return content transformed
        return HttpResponse(
            json.dumps(
                {
                    "template": render_xml_as_html_detail(
                        xml_content=xml_content,
                        template_id=template_id,
                        template_hash=template_hash,
                        xslt_id=xsl_transformation_id,
                        request=request,
                    ),
                }
            ),
            "application/javascript",
        )
    except AccessControlError:
        return HttpResponseForbidden("Access Forbidden")
    except Exception:
        return HttpResponseBadRequest("Unexpected error")



""" Url router for the explore OAI-PMH search application
"""

from django.conf.urls import include
from django.urls import re_path

import core_explore_oaipmh_app.views.user.ajax as user_ajax
import core_explore_oaipmh_app.views.user.views as user_views

urlpatterns = [
    re_path(
        r"^get_data_sources",
        user_ajax.get_data_source_list_oaipmh,
        name="core_explore_oaipmh_app_get_data_sources",
    ),
    re_path(
        r"^update_data_sources",
        user_ajax.update_data_source_list_oaipmh,
        name="core_explore_oaipmh_app_update_data_sources",
    ),
    re_path(
        r"^data",
        user_views.data_detail,
        name="core_explore_oaipmh_app_data_detail",
    ),
    re_path(
        r"^change-data-display",
        user_ajax.change_data_display,
        name="core_explore_oaipmh_app_change_data_display",
    ),
    re_path(r"^rest/", include("core_explore_oaipmh_app.rest.urls")),
]



""" REST views for the explore OAI-PMH API
"""

import json

import pytz
from django.conf import settings as conf_settings
from django.urls import reverse

import core_oaipmh_harvester_app.components.oai_record.api as oai_record_api
from core_explore_common_app.components.result.models import Result
from core_main_app.commons.constants import DATA_JSON_FIELD
from core_main_app.commons.exceptions import ApiError
from core_main_app.settings import RESULTS_PER_PAGE, DATA_SORTING_FIELDS
from core_main_app.utils.pagination.django_paginator.results_paginator import (
    ResultsPaginator,
)
from core_main_app.utils.pagination.mongoengine_paginator.paginator import (
    MongoenginePaginator,
)
from core_oaipmh_harvester_app.components.oai_harvester_metadata_format import (
    api as oai_harvester_metadata_format_api,
)
from core_oaipmh_harvester_app.components.oai_registry import (
    api as oai_registry_api,
)
from core_oaipmh_harvester_app.utils.query.mongo.query_builder import (
    OaiPmhQueryBuilder,
)


def get_template_info_from_metadata_format_and_template(
    harvester_metadata_format, template
):
    """Get template information from metadata format and template
    Args:
        harvester_metadata_format: Metadata format
        template: Related template

    Returns:
        Template information.

    """
    # Use the metadata prefix name
    name = harvester_metadata_format.get_display_name()

    # Here the id need to be set anyway because is expected by the serializer
    return_value = {
        "id": template.id if template is not None else "",
        "name": name,
        "hash": harvester_metadata_format.hash,
        "format": template.format,
    }

    return return_value


def get_registries(data):
    """Get a list of registry ids. Should return empty list if not found. JSON format.

    Returns:
        List of registry ids (JSON format).

    """
    registries_list = []
    options = data.get("options", None)
    if options is not None:
        if type(options) is str:  # Try parsing options only if it is a string
            options = json.loads(options)

        registries_list.append(int(options["instance_id"]))

    return json.dumps(registries_list)


def build_oaipmh_query(query_data):
    # get query and templates
    query = query_data.get("query", None)

    if query is None:
        raise ApiError("Query should be passed in parameter.")

    templates = query_data.get("templates", "[]")
    registries = get_registries(query_data)

    # build query builder
    query_builder = OaiPmhQueryBuilder(query, DATA_JSON_FIELD)

    if type(templates) is str:
        templates = json.loads(templates)

    if type(registries) is str:
        registries = json.loads(registries)

    # if registries, check if activated
    list_activated_registry = list(
        oai_registry_api.get_all_activated_registry().values_list(
            "id", flat=True
        )
    )
    if len(registries) > 0:
        activated_registries = [
            activated_registry_id
            for activated_registry_id in registries
            if activated_registry_id in list_activated_registry
        ]
    else:
        activated_registries = list_activated_registry

    if len(templates) > 0:
        # get list of template ids
        list_template_ids = [template["id"] for template in templates]
        # get all metadata formats used by the registries
        list_metadata_format = (
            oai_harvester_metadata_format_api.get_all_by_list_registry_ids(
                activated_registries
            )
        )
        # Filter metadata formats that use the given templates
        list_metadata_formats_id = [
            x.id
            for x in list_metadata_format
            if x.template is not None and x.template.id in list_template_ids
        ]
        query_builder.add_list_metadata_formats_criteria(
            list_metadata_formats_id
        )
    else:
        # Only activated registries
        query_builder.add_list_registries_criteria(activated_registries)

    # do not include deleted records
    query_builder.add_not_deleted_criteria()
    # return the raw query
    return query_builder.get_raw_query()


def execute_oaipmh_query(query_data, page, request):
    """Execute query on OAI-PMH database

    Args:
        query_data:
        page:
        request:


    Returns:

    """
    # build raw query
    raw_query = build_oaipmh_query(query_data)
    # retrieve order_by_field field
    order_by_field = query_data.get("order_by_field", None)
    order_by_field = (
        order_by_field.split(",") if order_by_field else DATA_SORTING_FIELDS
    )
    # execute query
    data_list = oai_record_api.execute_json_query(
        raw_query, request.user, order_by_field
    )
    # build result page
    if conf_settings.MONGODB_INDEXING:
        paginator = MongoenginePaginator(data_list, RESULTS_PER_PAGE)
        page = paginator.get_page(page)
    else:
        paginator = ResultsPaginator()
        page = paginator.get_results(data_list, page, RESULTS_PER_PAGE)
    return page


def format_oaipmh_results(results, request):
    """Format local results for explore apps

    Args:
        results:
        request:

    Returns:

    """
    url = reverse("core_explore_oaipmh_app_data_detail")
    url_access_data = reverse(
        "core_explore_oaipmh_app_rest_get_result_from_data_id"
    )
    # Template info
    template_info = dict()
    # Init data list
    data_list = []
    for data in results.object_list:
        # get data's metadata format
        metadata_format = data.harvester_metadata_format
        # get and store data's template information from metadata format
        if metadata_format not in template_info:
            template = data.harvester_metadata_format.template
            template_info[metadata_format] = (
                get_template_info_from_metadata_format_and_template(
                    metadata_format, template
                )
            )

        data_list.append(
            Result(
                title=data.title,
                content=data.xml_content,
                template_info=template_info[metadata_format],
                permission_url=None,
                detail_url=f"{url}?id={str(data.id)}",
                last_modification_date=data.last_modification_date.replace(
                    tzinfo=pytz.UTC
                ),
                access_data_url=f"{url_access_data}?id={str(data.id)}",
            )
        )
    return data_list
